/*  \brief Small application that transmits a scanned barcode directly after it is scanned by Bluetooth.
 *  \author Ronnie de Winter
 *  \author Michiel van den Brink
 *
 *  \version 1.0 Initial version
 *  \date    April 2011

 */

//Includes
#include <stdio.h>
#include <string.h>
#include "input.h"
#include "lib.h"
#include "msgscr.h"

//Defines
#define VERSION 				"XFF36680"
#define BCR_MAX  			 	50
#define BLT_MAX_PIN_LEN			16				// Maximum length of a Bluetooth PIN code
#define MAX_BARCODE_LENGTH      1000

#define BLT_MODE_NONE 	0
#define BLT_MODE_SSP 	1
#define BLT_MODE_HID 	2

//Global variables
int blt_hid_mode = BLT_MODE_NONE;

/*  Callback function that is called by the OS when the
 *  remote device request the entering of a PIN code.
 *  This function makes it possible to enter the PIN code
 *  using the numeric keys of the keyboard.
 *
 *  \param char *pin Pointer to the array where the input PIN nr should be stored.
 *  \param int* len  Length of the PIN array
 *
 *  \return OK All went well.
 */
int BltPinCallback( char *pin, int* len )
{
	char new_pin[BLT_MAX_PIN_LEN+1];

	for(;;)
	{
		printf("\fInsert PIN code\r\n");
		new_pin[0] = '\0';

		if(KeyboardInput( new_pin, 1, BLT_MAX_PIN_LEN, INPUT_NUM, 0, 1, 16, 1, ENT_KEY) == ENT_KEY)
			break;
	}

	strcpy( pin, new_pin );
	*len = strlen( pin );
	return 0;
}

/* This function tries to read a barcore from the scanner.
 * It will stop reader when either the scanner succeeds in reading a barcode.
 * Or when a timeout, a successful scan of trigger button is released.
 *
 * \param char* string			pointer to the array where the barcode will be put.
 * \param int min_length		Minimal length in characters to scan.
 * \param int max_length		Maximal length in characters to scan.
 * \param unsigned char timeout Timeout value in seconds
 *
 * \return 0  When a barcode has been read successful.
 * \return -1 Failed to read a barcode, due to time out or trigger release.
 */
int BltScanBarcode(char* string, int min_length, int max_length, unsigned char timeout)
{
	struct barcode code = {0};

	code.min = min_length;
	code.max = max_length;
	code.text = string;

	//Power on the scanner
	scannerpower( SINGLE, (timeout*50) ); //8 seconds timeout

	//Wait for the scanner to complete, either timeout or a successful scan of trigger button release.
	while( scanneroff() == 0 )
		idle();

	//Did the scanner succeed in reading a barcode?
	if(readbarcode( &code ) != OK)
		return ERROR;

	//Power off the scanner
	scannerpower( OFF, 0);

	return OK;
}

int transmit_barcode( char* barcode )
{
	putstring((unsigned char *)barcode);
	return OK;
}

void main( void )
{
	static char barcode[ MAX_BARCODE_LENGTH + 2+1 ];
	static char msg[MAX_BARCODE_LENGTH+25];
	static int need_to_transmit_barcode;
	int len;
	int ret;
	int com_port = 0;
	int connected = FALSE;
	static char options[54 + 1];	//Holds string of menu-options

	//Set the system settings.
	strcpy(options,"A0");			// Enable all codes
	strcat(options,"7G");			// show charging indication LED
	strcat(options,"*R");			// Handle shift key by application

	systemsetting( options );		//Execute all menu-options shown above

	autopowerdown( ON, 50*20 );		// 20 seconds

	InitMessageScreen();
	need_to_transmit_barcode = FALSE;

    //Set the callback to the request for a PIN number.
	SetBltPinCallback(BltPinCallback);

	//Print the introduction message.
	sprintf(barcode,"This %s application demonstrates the Bluetooth HID and SPP capabilities of the PX35", VERSION);
	ShowMessage(barcode, LANG_MSG_NL, MB_ALL | MB_ICONINFORMATION,0);
	
	if(ShowMessage("What kind of Bluetooth connection do you want?", LANG_MSG_NL, MB_ALL | MB_HIDSPP, 0 ) == NO )
		blt_hid_mode = BLT_MODE_HID;
	else
		blt_hid_mode = BLT_MODE_SSP;
	
	autopowerdown(ON,120*50);		// 2 minutes

	// The main application loop.
	for(;;)
	{
		//Ask for master of slave mode
		if(blt_hid_mode == BLT_MODE_HID)
			ret = ShowMessage("Do you want to setup a new Bluetooth HID connection?", LANG_MSG_NL, MB_ALL | MB_YESNO, 0 );
		else
			ret = ShowMessage("Do you want to connect as SPP Master?", LANG_MSG_NL, MB_ALL | MB_YESNO, 0 );

		if(blt_hid_mode != BLT_MODE_NONE )
		{
			if((blt_hid_mode == BLT_MODE_HID && ret == YES) || (blt_hid_mode == BLT_MODE_SSP && ret == NO)) //user wants to make a new connection
			{
				printf("\fWaiting for an\nincoming\nconnection...\nCLR to cancel.");

				//Select the correct comport for slave mode of HID or SPP
				if(blt_hid_mode == BLT_MODE_HID)
					com_port = COM12;	//COM12 			Bluetooth interface (Slave), using the HID profile
				else
					com_port = COM5;	//COM5  			Bluetooth interface (Slave), using the SPP profile
			}else //user wants to connect to an existing connection.
			{
				setfont( LARGE_FONT, NULL );
				printf("\fConnecting...\nCLR to cancel.");

				//Select the correct comport for master mode of HID or SPP
				if(blt_hid_mode == BLT_MODE_HID)
					com_port = COM13;	//COM13 			Bluetooth interface (Master), using the HID profile
				else
					com_port = COM3;	//COM3  			Bluetooth interface (Master), using the SPP profile

			}

			//Open the desired com port
			comopen(com_port);

			//Set the timeout timer to x seconds.
			starttimer(60*50);

			//Wait for a connection
			char c = 0;
			do{
				//If CLR is pressed cancel the connection
				if ((c = getchar()) ==  CLR_KEY)
					break;

				gotoxy(0,4);
				printf("Timeout in: %02d",timervalue()/50);
				idle();
			}
			while(blt_is_connected() != TRUE  && !endtimer());

			//If the timer expired of the user canceled the connection
			if((endtimer() || c == CLR_KEY) && blt_is_connected() != TRUE)
			{
				ShowMessage("Failed to connect to the remote device.", LANG_MSG_NL, MB_ALL | MB_ICONERROR,0 );
				connected = FALSE;
				continue; //Return to begin on loop
			}

			//The connection was a success.
			printf("\fConnected... ");
			connectionbeep();
			connected = TRUE;
		}
		else //Return to begin on loop
			continue;

		//This is the loop for when i'm connected!
		for(;;)
		{
			if( connected == TRUE && blt_is_connected() != TRUE) // Connection lost
			{
				comclose(com_port); //Close the previous opened comport.
				connected = FALSE;
				scannerpower( OFF, 0 ); //Disable the scanner

				ShowMessage("Connection is lost, press continue to reconnect.", LANG_MSG_NL, MB_ALL | MB_ICONERROR, 0);

				//If a connection was present, the device can reconnect in master mode even if it was in slave first.
				if(blt_hid_mode == BLT_MODE_HID)
					com_port = COM13; 	//COM13 			Bluetooth interface (Master), using the HID profile

				printf("\fReconnecting...\nCLR to cancel.");

				comopen(com_port);

				starttimer(30*50);

				//Wait for a connection
				char c = 0;
				do{
					//If CLR is pressed cancel the connection
					if ((c = getchar()) ==  CLR_KEY)
						break;

					idle();
					gotoxy(0,4);
					printf("Timeout in: %02d",timervalue()/50);
				}
				while(blt_is_connected() != TRUE  && !endtimer());

				//If the timer expired of the user canceled the connection
				if((endtimer() || c == CLR_KEY) && blt_is_connected() != TRUE)
				{
					ShowMessage("Reconnecting failed.", LANG_MSG_NL, MB_ALL | MB_ICONERROR,0 );
					connected = FALSE;
					break; //Break to the main for loop
				}

				//Reconnecting was a success!
				setfont( LARGE_FONT, NULL );
				printf("\fConnected...");
				connectionbeep();
				connected = TRUE;
			}
			
			if( connected == TRUE && need_to_transmit_barcode )
			{
				// A barcode was previously scanned but could not be transmitted
				if( ShowMessage("Do you want to resend the previous failed transmission", LANG_MSG_NL, MB_YESNO , 0 ) == YES )
				{
					if( transmit_barcode( barcode ) == OK )
					 	need_to_transmit_barcode = FALSE;
					else
					{
						if (com_port == COM13 || com_port == COM3)
							break; // connection failed
						else
							continue;
					}
				}
			}

			//Scan a barcode
			if(connected ==  TRUE)
			{
				gotoxy(0,1);
				printf("Scan barcode.");

				memset( barcode, '\0', sizeof( barcode ));
				//Scan a barcode
				if( BltScanBarcode(barcode, 1, 1000,2) == OK)
				{
					len = strlen(barcode);

					//Check if i'm able to transmit
					if (blt_is_connected() != TRUE)
					{
						goodreadled(RED, 10);
						sound(TLONG, VHIGH, SERROR, NULL);
						continue;
					}

					okbeep(); //Sound beep to indicate successful read

					//Print the barcode in the screen
					gotoxy(0,2);
					printf("%-14.14s\r\n%-14.14s", barcode, (strlen(barcode)>14) ? (barcode+14) : "" );

					need_to_transmit_barcode = TRUE;

					// Transmit the barcode
					if (blt_hid_mode == BLT_MODE_SSP)
						sprintf(msg,"%s\r", barcode);
					else
						sprintf(msg,"%s%c", barcode, 0xB2);		// 0xB2 is Enter key in PX35.

					if( need_to_transmit_barcode && transmit_barcode( msg ) == OK )
						need_to_transmit_barcode = FALSE;

				}
					if ((blt_hid_mode == BLT_MODE_HID) && (getchar() == '.'))
					{
							putcom ( 0xA6 ); // toggle iPhone on screen keyboard
					}
			}
		}

		// Close the Bluetooth port so that reconnection is possible
		comclose( com_port );
	}
}
